/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.AdminApplication");

dojo.require("com.ibm.team.repository.web.ui.internal._JazzApplication");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ProvisioningClient");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.PageContainer");

var _JazzApplication          = com.ibm.team.repository.web.ui.internal._JazzApplication;
var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var RepositoryClient 		= com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ProvisioningClient = com.ibm.team.repository.web.client.internal.ProvisioningClient;
var ProvisionResponseHandler = com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.AdminApplication", _JazzApplication, {
	_initUI: function() {
		dojo.addClass(document.body,"com-ibm-team-repository-web-admin-body");
		this.inherited(arguments);
	},
		
	init: function() {
		// fetch quick secured command to require ADMIN auth.  
		// can't use Util.userIsAdmin since it's false for everyone if DB is down
		var prh = new ProvisionResponseHandler(this, "_authRequestPassed", "_authRequestFailed");
		ProvisioningClient.isRepositoryAvailable(prh);		
	},	
	
	_fetchInitData: function (){
		this._initDataFetched = true;		
		var srh = new ServiceResponseHandler(this, "_handleInitData", "_fetchInitDataFailed");
		RepositoryClient.getInitializationData(srh);
	},
		
	_startApp: function(){
		var args = {
			label: "Admin Web UI",
			showLoginPanel: true
		};
		this._initUI(args);
	
		AdminWorkbench.superclass.start.call(this);
	},
	
	_authRequestPassed: function () {
		this._userIsAdmin = true;
		if (!this._initDataFetched)
			this._fetchInitData();
	}, 
	
	_authRequestFailed: function(errorObject) {
		if (errorObject.status === 403){
			this._userIsAdmin = false;
		}
		if (!this._initDataFetched)
			this._fetchInitData();
	}, 
		
	_handleInitData: function(initData) {
		InitializationData._load(initData);
		// handle case where there is no DB so contributor isn't in init data
		if (InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer") != null) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR", InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor);
		}
		// store the configuration properties in global vars with corresponding names
		var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer");
		for (var propName in configProperties) {
			var propVal = configProperties[propName];
			dojo.setObject(propName, propVal);
		}
		dojo.setObject("com.ibm.team.repository.web.pageorder", {"com.ibm.team.repository.web.admin":
	     			["com.ibm.team.repository.server"]});
		if (this._userIsAdmin == null || !this._userIsAdmin) {
			dojo.setObject("com.ibm.team.repository.web.suppressedPages", {
				"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.server", 
				"com.ibm.team.repository.provision", "com.ibm.team.process.ProjectAreaManagement", "com.ibm.team.repository.users"]
			});
			dojo.setObject("com.ibm.team.repository.web.noAdminAuth", "not authorized");
		} else {
/*
			if (com.ibm.team.repository.web.suppressedPages == null) {
				dojo.setObject("com.ibm.team.repository.web.suppressedPages", {
					"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.provision"]
				});
			} else if (com.ibm.team.repository.web.suppressedPages.com.ibm.team.repository.web.admin == null) {
				com.ibm.team.repository.web.suppressedPages.push({
					"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.provision"]
				});
			} else {
				com.ibm.team.repository.web.suppressedPages.com.ibm.team.repository.web.admin.push("com.ibm.team.repository.provision");
			}

*/			dojo.setObject("com.ibm.team.repository.web.suppressedPages", {
				"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.provision"]
			});
		}			
		this._startApp();
	},
		
	_fetchInitDataFailed: function(errorObject) {
		dojo.setObject("com.ibm.team.repository.web.pageorder", {"com.ibm.team.repository.web.admin":
	     			["com.ibm.team.repository.server", "com.ibm.team.repository.provision"]});
		if (this._userIsAdmin == null || !this._userIsAdmin) {
			dojo.setObject("com.ibm.team.repository.web.suppressedPages", {
				"com.ibm.team.repository.web.admin": ["com.ibm.team.repository.server", 
				"com.ibm.team.repository.provision", "com.ibm.team.process.ProjectAreaManagement", "com.ibm.team.repository.users"]
			});
			dojo.setObject("com.ibm.team.repository.web.noAdminAuth", "not authorized");
		}			
		this._startApp();
	}
	
});

var AdminWorkbench = com.ibm.team.repository.web.ui.internal.admin.AdminApplication;

})();
