/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters");

dojo.require("dojo.number");
dojo.require("dojox.collections.ArrayList");
dojo.require("dojox.collections.Set");
dojo.require("dojox.data.dom");
dojo.require("dojox.uuid.generateRandomUuid");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.internal.CountersClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

var Set = dojox.collections.Set;
var ArrayList = dojox.collections.ArrayList;

var CountersClient = com.ibm.team.repository.web.client.internal.CountersClient;
var CounterTable   = com.ibm.team.repository.web.ui.internal.admin.CounterTable;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
//var LightweightGenerator	= dojo.uuid.LightweightGenerator;

var ACTION_ID = "com.ibm.team.repository.admin.viewCounters";

// ATTRIBUTION: derived from Dojo 0.4.x dojo.string.endsWith
function endsWith(str, end) {
	if((str.length - end.length) < 0){
		return false; // boolean
	}
	return str.lastIndexOf(end) == str.length - end.length; // boolean
}

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ViewCounters.html"),

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters.superclass.postCreate.apply(this);
		this._hiddenGroups = new ArrayList();
		this._groupMap = {};
		this._fetchAndDisplayCounterInfo();
	},
	
	reload: function() {
		this._fetchAndDisplayCounterInfo();
	},
	
	show: function(args) {
		var hiddenGroups = args.hide ? args.hide.split(',') : [];
		this.setHiddenGroups(hiddenGroups);
	},
	
	_fetchAndDisplayCounterInfo: function() {
		var srh= new ServiceResponseHandler(this, "_displayCounterInfo", "_handleCounterFetchError");
		CountersClient.getAllCounters(srh);
	},

	_displayCounterInfo: function(counterInfo) {
		var _groups = counterInfo.groups;
			
		dojox.data.dom.removeChildren(this.countersArea);
		 
		if (_groups) {
			for (var i=0; i<_groups.length; i++) {
				var groupName = _groups[i].name;
				var args = {_counterWidget: this, _counterGroup: _groups[i]};
				var groupWidget = new com.ibm.team.repository.web.ui.internal.admin.CounterTable(args);
				if (this._hiddenGroups.contains(groupName)) {
					dojo.fx.wipeOut({ node: groupWidget.domNode, duration:1 }).play();
				}
				this.countersArea.appendChild(groupWidget.domNode);
				this._groupMap[groupName] = groupWidget;
			}
		}
		
		this._updateHideLinks();
	},
	
	_updateHideLinks: function() {
		// Build long hide list
		var baseHideRef = "#action=" + ACTION_ID + "&hide=";
		this._hiddenGroups.forEach(function(groupName) {
			baseHideRef += groupName + ","; 
		}, window);
		
		for (var group in this._groupMap) {
			this._groupMap[group].updateHideLink(baseHideRef + group);
		}
	}, 
	
	setHiddenGroups: function(groupNames) {
		// Hide newly hidden elements
		var newlyHidden = Set.difference(groupNames, this._hiddenGroups).toArray();
		var newlyHiddenDoms = [];
		for (var i=0;i<newlyHidden.length;i++) {
			var hiddenGroup = this._groupMap[newlyHidden[i]];
			if (hiddenGroup) {
				newlyHiddenDoms.push(hiddenGroup.domNode);
			}
		}
		if (newlyHiddenDoms.length > 0) {
			for (var i=0;i<newlyHiddenDoms.length;i++) {
				dojo.fx.wipeOut({ node: newlyHiddenDoms[i]}).play();
			}
		}
		
		var unhidden = Set.difference(this._hiddenGroups, groupNames).toArray();
		var unhiddenDoms = [];
		for (var i=0;i<unhidden.length;i++) {
			var unhiddenGroup = this._groupMap[unhidden[i]];
			if (unhiddenGroup) {
				unhiddenDoms.push(unhiddenGroup.domNode);
			}
		}
		if (unhiddenDoms.length > 0) {
			for (var i=0;i<unhiddenDoms.length;i++) {	
				dojo.fx.wipeIn({ node: unhiddenDoms[i]}).play();
			}
		}
		
		this._hiddenGroups = new ArrayList(groupNames);
		
		this._updateHideLinks();
	},
	
	_handleCounterFetchError: function(errorObj) {
		this._displayErrorMessage("Error fetching counter information", errorObj);
	},	

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			message += " (detailed message: " + 
				(errorObj.status === 0 ? "server unreachable - check network connectivity or contact server admin" : errorObj.message) +
				").";
		}
		this._displayMessage(message, "errorMsg");
	},
	
	// do not call directly
	_displayMessage: function(message, className) {
		dojo.dom.removeChildren(this.messageArea);
		this.messageArea.className= className;
		this.messageArea.appendChild(document.createTextNode(message));
	}

});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.CounterTable", [dijit._Widget, dijit._Templated], {
	
	templateString: "<div><h1 dojoAttachPoint='_heading'></h1><a dojoAttachPoint='_hideLink'>[hide]</a></div>",
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.CounterTable.superclass.postCreate.apply(this);
		
		this._heading.innerHTML = this._counterGroup.name;
		
		this.facetData = [];
		
		var groupTable= document.createElement("table");
		try {
			groupTable.className = "counter-table";
			
			// Add the header row for the facet names 
			var facetNamesRow = groupTable.insertRow(-1);
			facetNamesRow.className = "facet-names-row";
			facetNamesRow.insertCell(-1); // Add empty entry where the counter name will be
	
			// Add the header row for the facet stats 
			var facetStatsRow = groupTable.insertRow(-1);
			facetStatsRow.className = "facet-stats-row";
			facetStatsRow.insertCell(-1); // Add empty entry where the counter name will be
			
			if (this._counterGroup.counters) {
				var _counters = this._counterGroup.counters;
			
				for (var i=0; i<_counters.length; i++) {

					this._addCounter(groupTable, _counters[i]);
				}
			}

			this.domNode.appendChild(groupTable);
		} catch (e) {
			var errorDiv = document.createElement("div");
			var str = "";
			for (var prop in e) {
				str += prop + ": " + e[prop] + ", ";
			}
			errorDiv.appendChild(document.createTextNode("Error creating table: " + e.message));
			this.domNode.appendChild(errorDiv);
		}
	},
	
	updateHideLink: function(linkRef) {
		this._hideLink.href = linkRef;
	},
	
	_addCounter: function(counterTable, counter) {
		var counterRow = this._newCounterRow(counterTable, counter);

		if (counter.facets) {
			var _facets = counter.facets;
			for (var i=0; i<_facets.length; i++) {
				var facetData = this._getFacetData(counterTable, _facets[i]);
				
				// Insert the facet data into the cells
				this._updateFacetData(counterRow, facetData, _facets[i]);
			}
		}
	},
	
	_findStatCell: function(counterRow, facetData, statType) {
		var targetId = facetData.uuid + "-" + statType;
		var cells = counterRow.getElementsByTagName("td");
		for (var i=0; i<cells.length; i++) {
			var cell = cells[i];
			var idAttr = cell.id;
			if (idAttr && endsWith(idAttr, targetId)) {
				return cell;
			}
		}
		
		throw "The facet stat cell could not be found for '" + targetId + "'";		
	},
	
	_updateStatCell: function(counterRow, facetData, facet, statType, statValue, flags) {
		var statCell = this._findStatCell(counterRow, facetData, statType);
		dojox.data.dom.removeChildren(statCell);
		
		var cellText;
		if (statValue) {
			cellText = dojo.number.format(statValue, flags);
		}  else {
			cellText = "";
		}
		
		statCell.appendChild(document.createTextNode(cellText));
	},
	
	_updateFacetData: function(counterRow, facetData, facet) {
		if (facetData.type == "SCALAR") {
			flags = {places:0};
			this._updateStatCell(counterRow, facetData, facet, "val", facet.count, flags);
			this._updateStatCell(counterRow, facetData, facet, "max", facet.max, flags);
			this._updateStatCell(counterRow, facetData, facet, "min", facet.min, flags);
		} else {
			flags = {places:4};
			this._updateStatCell(counterRow, facetData, facet, "cnt", facet.count, {places:0});
			this._updateStatCell(counterRow, facetData, facet, "avg", facet.average, flags);
			this._updateStatCell(counterRow, facetData, facet, "max", facet.max, flags);
			this._updateStatCell(counterRow, facetData, facet, "min", facet.min, flags);
			this._updateStatCell(counterRow, facetData, facet, "dev", facet.stdv, flags);
			this._updateStatCell(counterRow, facetData, facet, "tot", facet.total, flags);
		}
	},
	
	_newCounterRow: function(counterTable, counter) {
		var newRowClass="counter-row";
		if (counterTable.firstChild.lastChild !== null) {
			if (counterTable.firstChild.lastChild.className != "counter-row-odd") {
				newRowClass="counter-row-odd";
			}
		}
		
		var counterRow = counterTable.insertRow(-1);
		counterRow.className = newRowClass;
		var nameCell = this._appendTableCell(counterRow, counter.name, {"className":"counter-name"});
		
		// Empty data slots need to be added for each
		// facet that already exists in this table
		for (var i=0; i<this.facetData.length; i++) {
			this._addEmptyFacetCells(this.facetData[i], counterRow);
		}
			
		return counterRow;
	},
	
	_getFacetData: function(counterTable, facet) {
		for (var i=0; i<this.facetData.length; i++) {
			if (this.facetData[i].name == facet.name) {
				// The facet has been seen before and can be returned
				return this.facetData[i];
			}
		} 
		
		// At this point the facet has not been seen before and
		// should be added to the list of facets
		
		var facetUUID = dojox.uuid.generateRandomUuid();
		
		// Mark the facet as having been seen to avoid
		// adding it twice
		var newFacetData = {
			type: facet.counterType,
			uuid: facetUUID,
			name: facet.name
		};
		this.facetData.push(newFacetData);

		this._addFacetHeaders(counterTable, newFacetData);
		
		// Now that the facet has been added
		// each row in the table needs to have the
		// facet cells added
		this._addFacetDataToRows(counterTable, newFacetData);
		
		return newFacetData;
	},
	
	_getFacetNamesRow: function(counterTable) {
		var rows = counterTable.getElementsByTagName("tr");
		for (var i=0; i<rows.length; i++) {
			if ("facet-names-row" == rows[i].className) {
				return rows[i];
			}
		}
		
		throw "The facet names row could not be found";
	},
	
	_getFacetStatsRow: function(counterTable) {
		var rows = counterTable.getElementsByTagName("tr");
		for (var i=0; i<rows.length; i++) {
			if ("facet-stats-row" == rows[i].className) {
				return rows[i];
			}
		}
		
		throw "The facet stats row could not be found";
	},	
	
	_appendTableCell: function(row, text, attributes) {

		var cell = row.insertCell(-1);
		cell.appendChild(document.createTextNode(text));
		
		
		if (attributes) {
			for (var attribute in attributes) {
				cell[attribute] = attributes[attribute];
			}
		}

	},
	
	_addFacetHeaders: function(counterTable, facetData) {
		var numDetails=1;
		var facetStatsRow = this._getFacetStatsRow(counterTable);
		this._appendTableCell(facetStatsRow, "", {"className":"spacer-cell"});
		if (facetData.type == "SCALAR") {
			numDetails += 3;
			this._appendTableCell(facetStatsRow, "val");
			this._appendTableCell(facetStatsRow, "max");
			this._appendTableCell(facetStatsRow, "min");
		} else {
			numDetails += 6;
			this._appendTableCell(facetStatsRow, "cnt");
			this._appendTableCell(facetStatsRow, "avg");
			this._appendTableCell(facetStatsRow, "max");
			this._appendTableCell(facetStatsRow, "min");
			this._appendTableCell(facetStatsRow, "dev");
			this._appendTableCell(facetStatsRow, "tot");
		}
		
		var facetNamesRow = this._getFacetNamesRow(counterTable);
		this._appendTableCell(facetNamesRow, facetData.name, {"colSpan": numDetails});
	},
	
	_addEmptyFacetCells: function(facetData, counterRow) {
		this._appendTableCell(counterRow, "", {"className":"spacer-cell"});		
		
		var uuid = "id_" + dojox.uuid.generateRandomUuid() + "-" + facetData.uuid; 

		if (facetData.type == "SCALAR") {
			this._appendTableCell(counterRow, "", {"id":uuid + "-val", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-max", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-min", "className":"data-cell"});
		} else {
			this._appendTableCell(counterRow, "", {"id":uuid + "-cnt", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-avg", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-max", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-min", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-dev", "className":"data-cell"});
			this._appendTableCell(counterRow, "", {"id":uuid + "-tot", "className":"data-cell"});
		}
	},
	
	_addFacetDataToRows: function(counterTable, facetData) {
		var rows = counterTable.getElementsByTagName("tr");
		for (var i = 0; i < rows.length; i++) { 
		    var rowClass = rows[i].className; 
		    if (rowClass.indexOf("counter-row") === 0) { 
				// Append the needed empty cells for the new facet
				this._addEmptyFacetCells(facetData, rows[i]);
		    }
		}
	}
});	

})();